import unittest

import numpy as np
from pymlfunc import sub2ind, ind2sub


def one_randint(limit=10):
    return np.random.randint(limit, size=1)[0]


class Test(unittest.TestCase):
    def setUp(self):
        nrows = one_randint(100)
        ncols = one_randint(100)
        nselected = one_randint(nrows)

        self.rows = np.random.randint(nrows, size=nselected)
        self.cols = np.random.randint(ncols, size=nselected)
        self.inds = np.random.randint(ncols * nrows, size=one_randint(nrows * ncols))

        self.mat = np.random.rand(nrows, ncols)

    def test_sub2ind(self):
        ind = sub2ind(self.mat.shape, self.rows, self.cols)
        self.assertTrue(np.allclose(self.mat[self.rows, self.cols], self.mat.ravel()[ind]))

    def test_ind2sub(self):
        rows, cols = ind2sub(self.mat.shape, self.inds)
        self.assertTrue(np.allclose(self.mat[rows, cols], self.mat.ravel()[self.inds]))



def sub2ind(array_shape, rows, cols):
    """
    Equivalent to the same-named function in Matlab (Except that Matlab is Fortran-styled indexing)
    Usage:
    In Matlab:
    A = reshape(1:30, 6, [])
    sub2ind(size(A), [2, 4], [1, 3])
    ans = 2    16

    Equivalently, in python:
    >>> mat = np.arange(1, 31).reshape((5, 6))
    >>> ind = sub2ind(mat.shape, np.array([0, 2]), np.array([1, 3]))
    >>> array([ 1, 15])

    Notice that the row and column arrays are swapped, and the indices are 1 less than that in Matlab,
    e.g. [1, 3] => [0, 2]

    :param array_shape:
    :param rows:
    :param cols:
    :return:
    """
    ind = rows * array_shape[1] + cols
    ind[ind < 0] = -1
    ind[ind >= array_shape[0] * array_shape[1]] = -1
    return ind


def ind2sub(array_shape, ind):
    """
    Equivalent to the same-named function in Matlab (Except that Matlab is Fortran-styled indexing)
    :param array_shape:
    :param rows:
    :param cols:
    :return:
    """
    ind[ind < 0] = -1
    ind[ind >= array_shape[0] * array_shape[1]] = -1
    rows = (ind.astype('int') // array_shape[1])
    cols = ind % array_shape[1]
    return rows, cols
